using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XELF;

namespace TileShader {

	public class TileGame : Microsoft.Xna.Framework.Game {
		readonly GraphicsDeviceManager graphics;

		const int TileWidth = 4;
		const int TileHeight = 4;
		const int MapWidth = 256;
		const int MapHeight = 256;

		readonly Color[] mapData = new Color[MapWidth * MapHeight];

		Effect effect;
		QuadFiller quad;
		Texture2D tile;
		Texture2D map;

		Vector3 position;
		float rotation;
		float scale = 0.125f * 64;
		Matrix transform = Matrix.Identity;

		public TileGame() {
			graphics = new GraphicsDeviceManager(this);
			graphics.PreferMultiSampling = true;
			graphics.PreferredBackBufferWidth = 1280;
			graphics.PreferredBackBufferHeight = 720;
			Content.RootDirectory = "Content";
		}

		protected override void LoadContent() {
			tile = Content.Load<Texture2D>("Tile");
			map = new Texture2D(GraphicsDevice, MapWidth, MapHeight, 0, TextureUsage.None, SurfaceFormat.Color);
			InitializeMap();
			map.SetData(mapData);
			quad = new QuadFiller(GraphicsDevice);
			effect = Content.Load<Effect>("TileEffect");
			effect.Parameters["tileTexture"].SetValue(tile);
			effect.Parameters["mapTexture"].SetValue(map);
			effect.Parameters["tileCells"].SetValue(new Vector2(TileWidth, TileHeight));
			effect.Parameters["mapCells"].SetValue(new Vector2(MapWidth, MapHeight));
			effect.Parameters["tileResolution"].SetValue(new Vector2(tile.Width, tile.Height));
		}

		void InitializeMap() {
			Random random = new Random();
			for (int i = 0; i < mapData.Length; i++) {
				mapData[i] = new Color((byte)random.Next(0, TileWidth), (byte)random.Next(0, TileHeight), 0, 0);
			}
		}

		protected override void Update(GameTime gameTime) {
			if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
				this.Exit();

			var state = Keyboard.GetState();
			if (state.IsKeyDown(Keys.A)) {
				rotation -= 0.01f;
			}
			if (state.IsKeyDown(Keys.D)) {
				rotation += 0.01f;
			}
			if (state.IsKeyDown(Keys.Q)) {
				scale *= 1.125f;
			}
			if (state.IsKeyDown(Keys.E)) {
				scale /= 1.125f;
			}

			var rotationMatrix = Matrix.CreateRotationZ(rotation);

			if (state.IsKeyDown(Keys.W)) {
				position += rotationMatrix.Down / 128.0f * scale;
			}
			if (state.IsKeyDown(Keys.S)) {
				position += rotationMatrix.Up / 128.0f * scale;
			}

			transform
				= Matrix.CreateScale(GraphicsDevice.Viewport.AspectRatio * scale, -scale, 1)
				* rotationMatrix
				* Matrix.CreateTranslation(position);

			effect.Parameters["transform"].SetValue(transform);

			base.Update(gameTime);
		}

		protected override void Draw(GameTime gameTime) {
			GraphicsDevice.Clear(Color.CornflowerBlue);
			quad.Fill(effect);
			base.Draw(gameTime);
		}
	}
}
